package eu.oskar3123.spigot2fa;

import eu.oskar3123.spigot2fa.command.TFACommand;
import eu.oskar3123.spigot2fa.config.PlayersConfiguration;
import eu.oskar3123.spigot2fa.config.SQLPlayersConfiguration;
import eu.oskar3123.spigot2fa.config.YAMLPlayersConfiguration;
import eu.oskar3123.spigot2fa.config.yaml.Config;
import eu.oskar3123.spigot2fa.config.yaml.ConfigHandler;
import eu.oskar3123.spigot2fa.handler.TFAHandler;
import eu.oskar3123.spigot2fa.listener.AuthListener;
import eu.oskar3123.spigot2fa.listener.MapListener;
import org.bukkit.configuration.Configuration;
import org.bukkit.plugin.PluginManager;
import org.bukkit.plugin.java.JavaPlugin;

import java.sql.SQLException;

public class Main extends JavaPlugin
{

    public TFAHandler tfaHandler;
    public ConfigHandler configHandler;
    public Config config;
    public PlayersConfiguration players;

    @Override
    public void onEnable()
    {
        this.configHandler = new ConfigHandler(this);
        this.config = this.configHandler.addConfig(new Config("config"));
        if (!initPlayersConfig())
        {
            getLogger().severe("Failed to init players backend, disabling plugin");
            getServer().getPluginManager().disablePlugin(this);
            return;
        }
        this.tfaHandler = new TFAHandler(this);
        registerCommands();
        registerListeners();
    }

    @Override
    public void onDisable()
    {
        if (this.tfaHandler != null)
        {
            this.tfaHandler.abortAll();
        }
    }

    private boolean initPlayersConfig()
    {
        Configuration c = this.configHandler.getConfig(this.config);
        String mode = c.getString("backend");
        if (mode.equalsIgnoreCase("yml") || mode.equalsIgnoreCase("yaml"))
        {
            players = new YAMLPlayersConfiguration(this.configHandler, "players");
            return true;
        }
        else if (mode.equalsIgnoreCase("mysql"))
        {
            String hostname = c.getString("mysql.hostname");
            String port = c.getString("mysql.port");
            String database = c.getString("mysql.database");
            String username = c.getString("mysql.username");
            String password = c.getString("mysql.password");
            String tablePrefix = c.getString("mysql.tableprefix");
            try
            {
                players = new SQLPlayersConfiguration(hostname, port, database, username, password, tablePrefix);
                return true;
            }
            catch (ClassNotFoundException e)
            {
                e.printStackTrace();
                getLogger().severe("JDBC driver for mysql not found");
                return false;
            }
            catch (SQLException e)
            {
                e.printStackTrace();
                getLogger().severe("Failed to connect to database, check your settings");
                return false;
            }
        }
        else
        {
            return false;
        }
    }

    private void registerCommands()
    {
        getCommand("2fa").setExecutor(new TFACommand(this));
    }

    private void registerListeners()
    {
        PluginManager pm = this.getServer().getPluginManager();
        pm.registerEvents(new AuthListener(this), this);
        pm.registerEvents(new MapListener(this), this);
    }

}
