package eu.oskar3123.spigot2fa.listener;

import eu.oskar3123.spigot2fa.Main;
import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.player.AsyncPlayerChatEvent;
import org.bukkit.event.player.PlayerCommandPreprocessEvent;
import org.bukkit.event.player.PlayerJoinEvent;
import org.bukkit.potion.PotionEffect;
import org.bukkit.potion.PotionEffectType;
import org.bukkit.scheduler.BukkitTask;

import java.util.*;

public class AuthListener implements Listener
{

    private class WaitingInfo
    {
        private BukkitTask kickTask;
        private BukkitTask teleportTask;

        private WaitingInfo(BukkitTask kickTask, BukkitTask teleportTask)
        {
            this.kickTask = kickTask;
            this.teleportTask = teleportTask;
        }

        private void cancel(Player player)
        {
            this.kickTask.cancel();
            this.teleportTask.cancel();
            player.removePotionEffect(PotionEffectType.BLINDNESS);
        }
    }

    private Main plugin;
    private Map<UUID, WaitingInfo> waitingForCode = new HashMap<>();

    public AuthListener(Main plugin)
    {
        this.plugin = plugin;
    }

    @EventHandler
    public void onJoin(PlayerJoinEvent event)
    {
        if (plugin.tfaHandler.shouldBypassCode(event.getPlayer()))
        {
            return;
        }
        final Player player = event.getPlayer();
        BukkitTask task = Bukkit.getScheduler().runTaskLater(plugin, () ->
        {
            waitingForCode.remove(player.getUniqueId());
            player.kickPlayer("You did not enter the 2FA code in time.");
        }, 800L);
        // 800 ticks == 40 secs
        final Location loginLocation = player.getLocation().clone();
        BukkitTask tpTask = Bukkit.getScheduler().runTaskTimer(plugin, () ->
        {
            player.teleport(loginLocation);
            player.addPotionEffect(new PotionEffect(PotionEffectType.BLINDNESS, 45, 0, false, false), true);
        }, 0L, 20L);
        waitingForCode.put(player.getUniqueId(), new WaitingInfo(task, tpTask));
        Bukkit.getScheduler().runTaskLater(plugin, () -> player.sendMessage("Please enter your 2FA code in the chat."), 1L);
    }

    @EventHandler
    public void chat(AsyncPlayerChatEvent event)
    {
        if (!waitingForCode.containsKey(event.getPlayer().getUniqueId()))
        {
            return;
        }
        boolean correct = plugin.tfaHandler.matchCode(event.getPlayer(), event.getMessage().replace(" ", ""));
        event.setCancelled(true);
        if (!correct)
        {
            event.getPlayer().sendMessage("Incorrect code, try again.");
        }
        else
        {
            waitingForCode.remove(event.getPlayer().getUniqueId()).cancel(event.getPlayer());
            plugin.tfaHandler.setLastLogin(event.getPlayer(), null);
            event.getPlayer().sendMessage("You have been authorized.");
        }
    }

    @EventHandler
    public void command(PlayerCommandPreprocessEvent event)
    {
        if (!waitingForCode.containsKey(event.getPlayer().getUniqueId()))
        {
            return;
        }
        event.setCancelled(true);
        event.getPlayer().sendMessage("You have to authorize first.");
    }

}
